import unittest

from charmhelpers.canonical_ci import nrpe

CONFIG = """
define service {
    use active-service
    host_name juju_jenkins-0
    service_description juju_jenkins-0 check_jenkins
    check_command check_nrpe!check_jenkins
    servicegroups services
}
"""


class NRPETests(unittest.TestCase):
    def test_http_check_no_params(self):
        check = nrpe.nrpe_check(
            check_type='http', name='check_my_webserver',
            hostname='webserver.com', port=80)
        ex = ("command[check_my_webserver]=/usr/lib/nagios"
              "/plugins/check_http --hostname=webserver.com --port=80")
        self.assertEquals(ex, check)

    def test_http_check_with_params(self):
        check = nrpe.nrpe_check(
            check_type='http', name='check_my_webserver',
            hostname='webserver.com', port=80, ip='10.0.0.3', pagesize=24)
        ex = ("command[check_my_webserver]=/usr/lib/nagios/"
              "plugins/check_http --hostname=webserver.com --port=80 "
              "--ip=10.0.0.3 --pagesize=24")
        self.assertEquals(ex, check)

    def test_tcp_check_no_params(self):
        check = nrpe.nrpe_check(
            check_type='tcp', name='check_gerrit',
            hostname='localhost', port=22)
        ex = ("command[check_gerrit]=/usr/lib/nagios/plugins/check_tcp"
              " --hostname=localhost --port=22")
        self.assertEquals(ex, check)

    def test_tcp_check_with_params(self):
        check = nrpe.nrpe_check(
            check_type='tcp', name='check_gerrit_tcp22',
            hostname='localhost', port=22, mismatch='ok', ssl=True)
        ex = ("command[check_gerrit_tcp22]=/usr/lib/nagios/plugins/check_tcp "
              "--hostname=localhost --port=22 --ssl --mismatch=ok")
        self.assertEquals(ex, check)

    def test_nagios_service_config(self):
        config = nrpe.nrpe_service_config(
            nagios_hostname='juju_jenkins-0', check_name='check_jenkins',
            nagios_servicegroup='services')
        self.assertEquals(CONFIG.strip(), config.strip())
